
/*
 * include important headers 
 */
#include <net-snmp/net-snmp-config.h>
#if HAVE_STDLIB_H
    #include <stdlib.h>
#endif
#if HAVE_STRING_H
    #include <string.h>
#else
    #include <strings.h>
#endif

/*
 * needed by util_funcs.h 
 */
#if TIME_WITH_SYS_TIME
    #ifdef WIN32
        #include <sys/timeb.h>
    #else
        #include <sys/time.h>
    #endif
    #include <time.h>
#else
    #if HAVE_SYS_TIME_H
        #include <sys/time.h>
    #else
        #include <time.h>
    #endif
#endif

#if HAVE_WINSOCK_H
    #include <winsock.h>
#endif
#if HAVE_NETINET_IN_H
    #include <netinet/in.h>
#endif

#include <net-snmp/net-snmp-includes.h>
#include <net-snmp/agent/net-snmp-agent-includes.h>



/*
 * header_generic() comes from here 
 */
#include "util_funcs.h"

/*
 * include our .h file 
 */
#include <iostream>
#include <vector>
#include "areca.h"
#include "arclib.h"
#include "linux_comm.h"
#include "linux_ioctl.h"

#define	ARECASTRING		    1
#define ARECAINTEGER		21
/*
 *  Certain objects can be set via configuration file directives.
 *  These variables hold the values for such objects, as they need to
 *   be accessible to both the config handlers, and the callback routine.
 */
#define ARECA_COMMUNITY_LEN	        256
#define MAX_CONTROLLER_SUPPORTED    4


CArclib ar[4];
static int nTotalCtrl = 0;
/*
 * This array defines the OID of the top of the mib tree that we're
 *  registering underneath.
 * Note that this needs to be the correct size for the OID being 
 *  registered, so that the length of the OID can be calculated.
 *  The format given here is the simplest way to achieve this.
 */
oid             areca_variables_oid[] = { 1, 3, 6, 1, 4, 1, 18928,1};

FILE *log_fd = NULL;    // debug only


#ifdef __cplusplus
extern "C" {
#endif

using namespace std;


enum
{
    EVT_DISABLE = 0,
    EVT_SERIOUS,        // 1 serious event
    EVT_ERROR,          // 2 error
    EVT_WARNING,        // 3 warning
    EVT_INFO            // 4 information
};

static unsigned char raidEventLevel[5] =
{
    EVT_WARNING, //RS_EVT_CREATE,
    EVT_WARNING, //RS_EVT_DELETE,
    EVT_WARNING, //RS_EVT_EXPAND,
    EVT_WARNING, //RS_EVT_REBUILD,
    EVT_SERIOUS  //RS_EVT_DEGRADED,
};

static unsigned char volEventLevel[14] =
{
    EVT_WARNING, //VS_EVT_INITIALIZING,
    EVT_WARNING, //VS_EVT_REBUILDING,
    EVT_WARNING, //VS_EVT_MIGRATING,
    EVT_WARNING, //VS_EVT_CHECKING,
    EVT_WARNING, //VS_EVT_COMPLETE_INIT,
    EVT_WARNING, //VS_EVT_COMPLETE_REBUILD,
    EVT_WARNING, //VS_EVT_COMPLETE_MIGRATING,
    EVT_WARNING, //VS_EVT_COMPLETE_CHECKING,
    EVT_WARNING, //VS_EVT_CREATE,
    EVT_WARNING, //VS_EVT_DELETE,
    EVT_WARNING, //VS_EVT_MODIFY,
    EVT_SERIOUS, //VS_EVT_DEGRADED,
    EVT_SERIOUS, //VS_EVT_FAILED,
    EVT_SERIOUS  //VS_EVT_REVIVED,
};

static unsigned char devEventLevel[13] =
{
    EVT_WARNING, //DEV_EVT_ADDED,
    EVT_WARNING, //DEV_EVT_REMOVED,
    EVT_WARNING, //DEV_EVT_READ_ERROR,
    EVT_WARNING, //DEV_EVT_WRITE_ERROR,
    EVT_WARNING, //DEV_EVT_ATA_ECC_ERROR,
    EVT_WARNING, //DEV_EVT_ATA_CHANGE_MODE,
    EVT_WARNING, //DEV_EVT_TIMEOUT,
    EVT_SERIOUS, //DEV_EVT_MARK_FAILED,
    EVT_ERROR,  //DEV_EVT_PCI_ERROR,
    EVT_SERIOUS, //DEV_EVT_SMART_FAILED,
    EVT_INFO,  //DEV_EVT_CREATE_PASS,
    EVT_INFO,  //DEV_EVT_MODIFY_PASS,
    EVT_INFO,  //DEV_EVT_DELETE_PASS,
};

static unsigned char hostEventLevel[3] =
{
    EVT_INFO, //SCSI_EVT_RESET,
    EVT_INFO, //SCSI_EVT_PARITY,
    EVT_INFO //SCSI_BUS_MODE_CHANGE,
};

static unsigned char hwEventLevel[40] =
{
    EVT_SERIOUS, //HW_EVT_SDRAM_1BIT_ECC,
    EVT_SERIOUS, //HW_EVT_SDRAM_MULTI_BIT_ECC,
    EVT_SERIOUS, //HW_EVT_TEMP_CONTROLLER,
    EVT_SERIOUS, //HW_EVT_TEMP_BACKPLANE,
    EVT_SERIOUS, //HW_EVT_VOLTAGE15,
    EVT_SERIOUS, //HW_EVT_VOLTAGE3,
    EVT_SERIOUS, //HW_EVT_VOLTAGE5,
    EVT_SERIOUS, //HW_EVT_VOLTAGE12,
    EVT_SERIOUS, //HW_EVT_VOLTAGE1_3,
    EVT_SERIOUS, //HW_EVT_VOLTAGE2_5,
    EVT_SERIOUS, //HW_EVT_VOLTAGE1_25,
    EVT_SERIOUS, //HW_EVT_POWER1_FAILED,
    EVT_SERIOUS, //HW_EVT_FAN1_FAILED,
    EVT_SERIOUS, //HW_EVT_POWER2_FAILED,
    EVT_SERIOUS, //HW_EVT_FAN2_FAILED,
    EVT_SERIOUS, //HW_EVT_POWER3_FAILED,
    EVT_SERIOUS, //HW_EVT_FAN3_FAILED,
    EVT_SERIOUS, //HW_EVT_POWER4_FAILED,
    EVT_SERIOUS, //HW_EVT_FAN4_FAILED,
    EVT_SERIOUS, //HW_EVT_UPS_POWER_LOSS,
    // Add to log H/W monitor recovered
    EVT_ERROR,  //HW_EVT_TEMP_CONTROLLER_R,
    EVT_ERROR,  //HW_EVT_TEMP_BACKPLANE_R,
    EVT_ERROR,  //HW_EVT_VOLTAGE15_R,
    EVT_ERROR,  //HW_EVT_VOLTAGE3_R,
    EVT_ERROR,  //HW_EVT_VOLTAGE5_R,
    EVT_ERROR,  //HW_EVT_VOLTAGE12_R,
    EVT_ERROR,  //HW_EVT_VOLTAGE1_3_R,
    EVT_ERROR,  //HW_EVT_VOLTAGE2_5_R,
    EVT_ERROR,  //HW_EVT_VOLTAGE1_25_R,
    EVT_ERROR,  //HW_EVT_POWER1_RECOVERED,
    EVT_ERROR,  //HW_EVT_FAN1_RECOVERED,
    EVT_ERROR,  //HW_EVT_POWER2_RECOVERED,
    EVT_ERROR,  //HW_EVT_FAN2_RECOVERED,
    EVT_ERROR,  //HW_EVT_POWER3_RECOVERED,
    EVT_ERROR,  //HW_EVT_FAN3_RECOVERED,
    EVT_ERROR,  //HW_EVT_POWER4_RECOVERED,
    EVT_ERROR,  //HW_EVT_FAN4_RECOVERED,
    EVT_ERROR,  //HW_EVT_UPS_POWER_RECOVERED,

    EVT_WARNING, //HW_EVT_SYSTEM_RESTARTED,
    EVT_SERIOUS  //HW_EVT_TEST_EVENT
};


static unsigned char newEventLevel[14] =
{
    EVT_ERROR,      // "Recovered"
    EVT_SERIOUS,    // "Over Temp."
    EVT_SERIOUS,    // "Failed"
    EVT_SERIOUS,    // "Under Voltage"
    EVT_SERIOUS,    // "Over Voltage"
    EVT_WARNING,    // reserved
    EVT_WARNING,    // reserved
    EVT_WARNING,    // reserved
    EVT_ERROR,      // "HTTP Log In",
    EVT_ERROR,      // "Telnet Log In"
    EVT_ERROR,      // "VT100 Log In"
    EVT_ERROR,      // "API Log In"
    EVT_ERROR,      // "FC Link Up"
    EVT_ERROR       // "Fc Link Down"
};

/*
 * Forward declarations for the config handlers 
 */
void            areca_parse_config_rocommunity(const char *token,
                                               char *cptr);
void            areca_parse_config_rwcommunity(const char *token,
                                               char *cptr);
void            areca_parse_config_evtlevel(const char *token, char *cptr);

void            EventMonitor(unsigned int clientreg, void *clientarg);



// the default community and event notification level
char gCommunity[ARECA_COMMUNITY_LEN] = "public";
int  gEvt_level = 0;

/*********************
*
*  Initialisation & common implementation functions
*
*********************/

/*
 * This array structure defines a representation of the
 *  MIB being implemented.
 *
 * The type of the array is 'struct variableN', where N is
 *  large enough to contain the longest OID sub-component
 *  being loaded.  This will normally be the maximum value
 *  of the fifth field in each line.  In this case, the second
 *  and third entries are both of size 2, so we're using
 *  'struct variable2'
 *
 * The supported values for N are listed in <agent/var_struct.h>
 *  If the value you need is not listed there, simply use the
 *  next largest that is.
 *
 * The format of each line is as follows
 *  (using the first entry as an areca):
 *      1: ARECASTRING:
 *          The magic number defined in the areca header file.
 *          This is passed to the callback routine and is used
 *            to determine which object is being queried.
 *      2: ASN_OCTET_STR:
 *          The type of the object.
 *          Valid types are listed in <snmp_impl.h>
 *      3: RONLY (or RWRITE):
 *          Whether this object can be SET or not.
 *      4: var_areca:
 *          The callback routine, used when the object is queried.
 *          This will usually be the same for all objects in a module
 *            and is typically defined later in this file.
 *      5: 1:
 *          The length of the OID sub-component (the next field)
 *      6: {1}:
 *          The OID sub-components of this entry.
 *          In other words, the bits of the full OID that differ
 *            between the various entries of this array.
 *          This value is appended to the common prefix (defined later)
 *            to obtain the full OID of each entry.
 */


struct variable7 areca_variables1[] = {

    // system information
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 3, {1, 1, 1}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 3, {1, 1, 2}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 3, {1, 1, 3}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 3, {1, 1, 4}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 3, {1, 1, 5}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 3, {1, 1, 6}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 3, {1, 1, 7}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {1, 1, 8}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {1, 1, 9}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {1, 1, 10}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {1, 1, 11}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {1, 1, 12}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {1, 1, 13}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {1, 1, 14}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {1, 1, 15}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {1, 1, 16}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {1, 1, 17}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {1, 1, 18}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {1, 1, 19}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {1, 1, 20}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {1, 1, 21}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {1, 1, 22}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {1, 1, 23}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {1, 1, 24}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {1, 1, 25}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {1, 1, 26}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {1, 1, 27}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {1, 1, 28}},   

    // hw monitor
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {1, 2, 1}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {1, 2, 2}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {1, 2, 3}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {1, 2, 4}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {1, 2, 5}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {1, 2, 6}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {1, 2, 7}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {1, 2, 8}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {1, 2, 9}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {1, 2, 10}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {1, 2, 11}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {1, 2, 12, 1, 1}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {1, 2, 12, 1, 2}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {1, 2, 13, 1, 1}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {1, 2, 13, 1, 2}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {1, 2, 14, 1, 1}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {1, 2, 14, 1, 2}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 3, {1, 2, 15}},

    // hdd information
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {1, 3, 1, 1, 1}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 5, {1, 3, 1, 1, 2}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 5, {1, 3, 1, 1, 3}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 5, {1, 3, 1, 1, 4}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {1, 3, 1, 1, 5}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {1, 3, 1, 1, 6}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {1, 3, 1, 1, 7}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {1, 3, 1, 1, 8}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 5, {1, 3, 1, 1, 9}},

    // raidset information
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {1, 4, 1, 1, 1}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 5, {1, 4, 1, 1, 2}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {1, 4, 1, 1, 3}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 5, {1, 4, 1, 1, 4}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {1, 4, 1, 1, 5}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {1, 4, 1, 1, 6}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {1, 4, 1, 1, 7}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 5, {1, 4, 1, 1, 8}},

    // volumeset information
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {1, 5, 1, 1, 1}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 5, {1, 5, 1, 1, 2}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 5, {1, 5, 1, 1, 3}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {1, 5, 1, 1, 4}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 5, {1, 5, 1, 1, 5}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {1, 5, 1, 1, 6}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {1, 5, 1, 1, 7}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {1, 5, 1, 1, 8}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {1, 5, 1, 1, 9}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {1, 5, 1, 1, 10}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {1, 5, 1, 1, 11}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {1, 5, 1, 1, 12}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {1, 5, 1, 1, 13}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {1, 5, 1, 1, 14}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {1, 5, 1, 1, 15}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 5, {1, 5, 1, 1, 16}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 5, {1, 5, 1, 1, 17}},

    // event information
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {1, 6, 1, 1, 1}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 5, {1, 6, 1, 1, 2}},

    // trap information
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 128}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 129}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 130}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 131}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 132}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 133}},

    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 256}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 257}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 258}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 259}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 260}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 261}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 262}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 263}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 264}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 265}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 266}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 267}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 268}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 269}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 270}},

    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 384}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 385}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 386}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 387}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 388}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 389}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 390}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 391}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 392}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 393}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 394}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 395}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 396}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 397}},

    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 512}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 513}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 514}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 515}},

    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 640}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 641}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 642}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 643}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 644}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 645}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 646}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 647}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 648}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 649}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 650}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 651}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 652}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 653}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 654}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 655}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 656}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 657}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 658}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 659}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 660}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 661}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 662}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 663}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 664}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 665}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 666}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 667}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 668}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 669}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 670}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 671}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 672}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 673}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 674}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 675}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {1, 0, 676}}
};

struct variable7 areca_variables2[] = {

    // system information
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 3, {2, 1, 1}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 3, {2, 1, 2}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 3, {2, 1, 3}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 3, {2, 1, 4}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 3, {2, 1, 5}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 3, {2, 1, 6}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 3, {2, 1, 7}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {2, 1, 8}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {2, 1, 9}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {2, 1, 10}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {2, 1, 11}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {2, 1, 12}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {2, 1, 13}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {2, 1, 14}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {2, 1, 15}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {2, 1, 16}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {2, 1, 17}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {2, 1, 18}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {2, 1, 19}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {2, 1, 20}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {2, 1, 21}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {2, 1, 22}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {2, 1, 23}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {2, 1, 24}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {2, 1, 25}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {2, 1, 26}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {2, 1, 27}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {2, 1, 28}},   

    // hw monitor
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {2, 2, 1}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {2, 2, 2}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {2, 2, 3}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {2, 2, 4}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {2, 2, 5}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {2, 2, 6}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {2, 2, 7}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {2, 2, 8}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {2, 2, 9}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {2, 2, 10}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {2, 2, 11}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {2, 2, 12, 1, 1}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {2, 2, 12, 1, 2}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {2, 2, 13, 1, 1}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {2, 2, 13, 1, 2}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {2, 2, 14, 1, 1}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {2, 2, 14, 1, 2}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 3, {2, 2, 15}},

    // hdd information
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {2, 3, 1, 1, 1}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 5, {2, 3, 1, 1, 2}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 5, {2, 3, 1, 1, 3}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 5, {2, 3, 1, 1, 4}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {2, 3, 1, 1, 5}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {2, 3, 1, 1, 6}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {2, 3, 1, 1, 7}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {2, 3, 1, 1, 8}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 5, {2, 3, 1, 1, 9}},

    // raidset information
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {2, 4, 1, 1, 1}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 5, {2, 4, 1, 1, 2}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {2, 4, 1, 1, 3}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 5, {2, 4, 1, 1, 4}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {2, 4, 1, 1, 5}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {2, 4, 1, 1, 6}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {2, 4, 1, 1, 7}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 5, {2, 4, 1, 1, 8}},

    // volumeset information
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {2, 5, 1, 1, 1}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 5, {2, 5, 1, 1, 2}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 5, {2, 5, 1, 1, 3}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {2, 5, 1, 1, 4}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 5, {2, 5, 1, 1, 5}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {2, 5, 1, 1, 6}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {2, 5, 1, 1, 7}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {2, 5, 1, 1, 8}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {2, 5, 1, 1, 9}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {2, 5, 1, 1, 10}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {2, 5, 1, 1, 11}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {2, 5, 1, 1, 12}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {2, 5, 1, 1, 13}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {2, 5, 1, 1, 14}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {2, 5, 1, 1, 15}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 5, {2, 5, 1, 1, 16}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 5, {2, 5, 1, 1, 17}},

    // event information
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {2, 6, 1, 1, 1}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 5, {2, 6, 1, 1, 2}},

    // trap information
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 128}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 129}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 130}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 131}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 132}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 133}},

    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 256}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 257}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 258}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 259}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 260}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 261}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 262}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 263}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 264}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 265}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 266}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 267}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 268}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 269}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 270}},

    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 384}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 385}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 386}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 387}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 388}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 389}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 390}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 391}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 392}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 393}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 394}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 395}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 396}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 397}},

    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 512}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 513}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 514}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 515}},

    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 640}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 641}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 642}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 643}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 644}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 645}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 646}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 647}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 648}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 649}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 650}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 651}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 652}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 653}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 654}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 655}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 656}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 657}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 658}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 659}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 660}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 661}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 662}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 663}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 664}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 665}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 666}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 667}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 668}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 669}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 670}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 671}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 672}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 673}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 674}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 675}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {2, 0, 676}}
};

struct variable7 areca_variables3[] = {

    // system information
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 3, {3, 1, 1}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 3, {3, 1, 2}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 3, {3, 1, 3}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 3, {3, 1, 4}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 3, {3, 1, 5}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 3, {3, 1, 6}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 3, {3, 1, 7}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {3, 1, 8}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {3, 1, 9}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {3, 1, 10}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {3, 1, 11}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {3, 1, 12}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {3, 1, 13}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {3, 1, 14}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {3, 1, 15}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {3, 1, 16}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {3, 1, 17}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {3, 1, 18}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {3, 1, 19}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {3, 1, 20}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {3, 1, 21}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {3, 1, 22}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {3, 1, 23}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {3, 1, 24}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {3, 1, 25}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {3, 1, 26}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {3, 1, 27}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {3, 1, 28}},   

    // hw monitor
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {3, 2, 1}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {3, 2, 2}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {3, 2, 3}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {3, 2, 4}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {3, 2, 5}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {3, 2, 6}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {3, 2, 7}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {3, 2, 8}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {3, 2, 9}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {3, 2, 10}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {3, 2, 11}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {3, 2, 12, 1, 1}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {3, 2, 12, 1, 2}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {3, 2, 13, 1, 1}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {3, 2, 13, 1, 2}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {3, 2, 14, 1, 1}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {3, 2, 14, 1, 2}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 3, {3, 2, 15}},

    // hdd information
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {3, 3, 1, 1, 1}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 5, {3, 3, 1, 1, 2}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 5, {3, 3, 1, 1, 3}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 5, {3, 3, 1, 1, 4}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {3, 3, 1, 1, 5}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {3, 3, 1, 1, 6}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {3, 3, 1, 1, 7}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {3, 3, 1, 1, 8}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 5, {3, 3, 1, 1, 9}},

    // raidset information
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {3, 4, 1, 1, 1}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 5, {3, 4, 1, 1, 2}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {3, 4, 1, 1, 3}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 5, {3, 4, 1, 1, 4}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {3, 4, 1, 1, 5}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {3, 4, 1, 1, 6}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {3, 4, 1, 1, 7}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 5, {3, 4, 1, 1, 8}},

    // volumeset information
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {3, 5, 1, 1, 1}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 5, {3, 5, 1, 1, 2}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 5, {3, 5, 1, 1, 3}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {3, 5, 1, 1, 4}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 5, {3, 5, 1, 1, 5}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {3, 5, 1, 1, 6}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {3, 5, 1, 1, 7}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {3, 5, 1, 1, 8}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {3, 5, 1, 1, 9}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {3, 5, 1, 1, 10}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {3, 5, 1, 1, 11}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {3, 5, 1, 1, 12}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {3, 5, 1, 1, 13}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {3, 5, 1, 1, 14}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {3, 5, 1, 1, 15}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 5, {3, 5, 1, 1, 16}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 5, {3, 5, 1, 1, 17}},

    // event information
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {3, 6, 1, 1, 1}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 5, {3, 6, 1, 1, 2}},

    // trap information
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 128}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 129}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 130}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 131}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 132}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 133}},

    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 256}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 257}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 258}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 259}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 260}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 261}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 262}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 263}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 264}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 265}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 266}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 267}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 268}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 269}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 270}},

    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 384}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 385}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 386}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 387}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 388}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 389}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 390}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 391}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 392}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 393}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 394}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 395}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 396}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 397}},

    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 512}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 513}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 514}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 515}},

    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 640}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 641}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 642}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 643}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 644}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 645}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 646}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 647}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 648}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 649}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 650}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 651}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 652}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 653}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 654}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 655}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 656}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 657}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 658}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 659}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 660}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 661}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 662}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 663}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 664}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 665}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 666}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 667}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 668}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 669}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 670}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 671}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 672}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 673}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 674}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 675}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {3, 0, 676}}
};

struct variable7 areca_variables4[] = {

    // system information
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 3, {4, 1, 1}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 3, {4, 1, 2}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 3, {4, 1, 3}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 3, {4, 1, 4}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 3, {4, 1, 5}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 3, {4, 1, 6}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 3, {4, 1, 7}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {4, 1, 8}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {4, 1, 9}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {4, 1, 10}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {4, 1, 11}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {4, 1, 12}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {4, 1, 13}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {4, 1, 14}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {4, 1, 15}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {4, 1, 16}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {4, 1, 17}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {4, 1, 18}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {4, 1, 19}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {4, 1, 20}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {4, 1, 21}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {4, 1, 22}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {4, 1, 23}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {4, 1, 24}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {4, 1, 25}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {4, 1, 26}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {4, 1, 27}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {4, 1, 28}},   

    // hw monitor
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {4, 2, 1}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {4, 2, 2}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {4, 2, 3}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {4, 2, 4}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {4, 2, 5}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {4, 2, 6}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {4, 2, 7}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {4, 2, 8}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {4, 2, 9}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {4, 2, 10}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 3, {4, 2, 11}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {4, 2, 12, 1, 1}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {4, 2, 12, 1, 2}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {4, 2, 13, 1, 1}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {4, 2, 13, 1, 2}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {4, 2, 14, 1, 1}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {4, 2, 14, 1, 2}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 3, {4, 2, 15}},

    // hdd information
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {4, 3, 1, 1, 1}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 5, {4, 3, 1, 1, 2}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 5, {4, 3, 1, 1, 3}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 5, {4, 3, 1, 1, 4}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {4, 3, 1, 1, 5}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {4, 3, 1, 1, 6}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {4, 3, 1, 1, 7}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {4, 3, 1, 1, 8}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 5, {4, 3, 1, 1, 9}},

    // raidset information
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {4, 4, 1, 1, 1}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 5, {4, 4, 1, 1, 2}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {4, 4, 1, 1, 3}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 5, {4, 4, 1, 1, 4}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {4, 4, 1, 1, 5}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {4, 4, 1, 1, 6}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {4, 4, 1, 1, 7}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 5, {4, 4, 1, 1, 8}},

    // volumeset information
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {4, 5, 1, 1, 1}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 5, {4, 5, 1, 1, 2}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 5, {4, 5, 1, 1, 3}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {4, 5, 1, 1, 4}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 5, {4, 5, 1, 1, 5}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {4, 5, 1, 1, 6}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {4, 5, 1, 1, 7}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {4, 5, 1, 1, 8}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {4, 5, 1, 1, 9}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {4, 5, 1, 1, 10}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {4, 5, 1, 1, 11}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {4, 5, 1, 1, 12}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {4, 5, 1, 1, 13}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {4, 5, 1, 1, 14}},
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {4, 5, 1, 1, 15}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 5, {4, 5, 1, 1, 16}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 5, {4, 5, 1, 1, 17}},

    // event information
    {ARECAINTEGER, ASN_INTEGER, RONLY, var_areca, 5, {4, 6, 1, 1, 1}},
    {ARECASTRING, ASN_OCTET_STR, RONLY, var_areca, 5, {4, 6, 1, 1, 2}},

    // trap information
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 128}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 129}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 130}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 131}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 132}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 133}},

    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 256}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 257}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 258}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 259}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 260}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 261}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 262}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 263}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 264}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 265}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 266}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 267}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 268}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 269}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 270}},

    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 384}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 385}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 386}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 387}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 388}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 389}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 390}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 391}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 392}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 393}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 394}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 395}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 396}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 397}},

    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 512}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 513}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 514}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 515}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 640}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 641}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 642}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 643}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 644}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 645}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 646}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 647}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 648}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 649}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 650}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 651}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 652}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 653}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 654}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 655}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 656}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 657}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 658}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 659}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 660}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 661}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 662}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 663}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 664}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 665}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 666}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 667}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 668}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 669}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 670}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 671}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 672}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 673}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 674}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 675}},
    {ARECAINTEGER, ASN_OBJECT_ID, NOACCESS, var_areca, 3, {4, 0, 676}}
};

char *arctime(const struct tm *timeptr)
{
    static char result[26];

    memset(result, 0, sizeof(result));

    sprintf(result, "%4d-%02d-%02d %02d:%02d:%02d", 
            timeptr->tm_year+1900,
            timeptr->tm_mon,
            timeptr->tm_mday,
            timeptr->tm_hour,
            timeptr->tm_min,
            timeptr->tm_sec);

    return result;
}


void
DebugPrint(FILE *fd, char *fmtstr, ...)
{

    time_t now;
    char bufx[128];
    char bufferline[256];


    time(&now);
    memset(bufx, 0, sizeof(bufx));
    memset(bufferline, 0, sizeof(bufferline));
    va_list argptr;
    va_start(argptr, fmtstr);
    vsnprintf(bufx, sizeof(bufx), fmtstr, argptr);
    va_end(argptr);    
    sprintf(bufferline, "%s %s", arctime(localtime(&now)), bufx);

    fprintf(fd, bufferline);
    fflush(fd);

}


int DiscoveryDevice()
{
    char buf[64];
    char rmstr[128];
    mode_t mode = 0666;
    dev_t dev;
    int     nNumofPciDevs = 0;

    mode |= S_IFCHR;


    // remove all exist device node first
    for (int i = 0; i < 4; i++)
    {
        memset(buf, 0, sizeof(buf));
        sprintf(buf,"/dev/arcmsr%d", i);
        unlink(buf);
    }

	
    LinuxIoctlInterface *out = new LinuxIoctlInterface();

    for (int mymajor = 255; mymajor > 161; mymajor--)
    {
        for (int myminor = 0; myminor < 4; myminor++)
        {
            memset(buf, 0, sizeof(buf));
            sprintf(buf,"/dev/arcmsr%d", nTotalCtrl);
            dev = makedev(mymajor, myminor);
            mknod(buf, mode, dev);
            if (out->init(nTotalCtrl))
            {
                ar[nTotalCtrl++].ArcInitSession(out);
                out = new LinuxIoctlInterface();
            }
            else
            {
                unlink(buf);
            }
        }


        if (nTotalCtrl)
        {
            break;
        }
    }

    return nTotalCtrl;

}



/*
 * This function is called at the time the agent starts up
 *  to do any initializations that might be required.
 *
 * In theory it is optional and can be omitted if no
 *  initialization is needed.  In practise, every module
 *  will need to register itself (or the objects being
 *  implemented will not appear in the MIB tree), and this
 *  registration is typically done here.
 *
 * If this function is added or removed, you must re-run
 *  the configure script, to detect this change.
 */
void
init_areca(void)
{


    log_fd = fopen("/var/log/arcsnmp.log", "w+");


    DebugPrint(log_fd, "init_areca()\r\n");

    int ctrl = DiscoveryDevice();
    DebugPrint(log_fd, "[%d] controller found\r\n", ctrl);
    if ( ctrl == 0 )
    {    
        return;
    }

    /*
     * Register ourselves with the agent to handle our mib tree.
     * The arguments are:
     *    descr:   A short description of the mib group being loaded.
     *    var:     The variable structure to load.
     *                  (the name of the variable structure defined above)
     *    vartype: The type of this variable structure
     *    theoid:  The OID pointer this MIB is being registered underneath.
     */

    REGISTER_MIB("areca", areca_variables1, variable7,
                 areca_variables_oid);

    REGISTER_MIB("areca", areca_variables2, variable7,
                 areca_variables_oid);

    REGISTER_MIB("areca", areca_variables3, variable7,
                 areca_variables_oid);

    REGISTER_MIB("areca", areca_variables4, variable7,
                 areca_variables_oid);

    DebugPrint(log_fd, "MIB Registered OK.\r\n");

    // ignore the old events
    int nCount = 0;
    for ( int index = 0; index < ctrl; index++ )
    {
        ar[index].ArcGetReqEventPage(FLAG_ALL, &nCount);
    }

    snmp_alarm_register(1, SA_REPEAT, EventMonitor, NULL);

    DebugPrint(log_fd, "snmpd_alarm_registered OK\r\n");

    register_config_handler("arcsnmp", "community", areca_parse_config_rocommunity, NULL, "raidctrl snmp community");
    register_config_handler("arcsnmp", "evtlevel", areca_parse_config_evtlevel, NULL, "raidctrl event notification level");

}

/*********************
*
*  Configuration file handling functions
*
*********************/

void
areca_parse_config_rocommunity(const char *token, char *cptr)
{
    /*
     * Make sure the string fits in the space allocated for it.
     */  

    int index = 0;
    char *ch = cptr;

    if ( cptr == NULL )
        return;

    if ( !strlen(cptr) )
        return;

    strcpy((char *)gCommunity, cptr);


    gCommunity[strlen(cptr)] = '\0';


    DebugPrint(log_fd, "rocommunity = %s\r\n", gCommunity);
}

void
areca_parse_config_rwcommunity(const char *token, char *cptr)
{
    /*
     * Make sure the string fits in the space allocated for it.
     */  

    int index = 0;
    char *ch = cptr;


    if ( cptr == NULL )
        return;

    if ( !strlen(cptr) )
        return;

    strcpy((char *)gCommunity, cptr);

    gCommunity[strlen(cptr)] = '\0';

    DebugPrint(log_fd, "rwcommunity = %s\r\n", gCommunity);
}

void
areca_parse_config_evtlevel(const char *token, char *cptr)
{
    if ( cptr != NULL )
    {
        gEvt_level = atoi(cptr);
    }

    DebugPrint(log_fd, "event level = %d\r\n", gEvt_level);
}


char *SnmpUtilIdsToA(oid *name, size_t * length, char *gOidBuf)
{
    char tmp[16];

    memset(gOidBuf, 0, sizeof(gOidBuf));

    if ( name == NULL || length == NULL )
    {
        return NULL;
    }

    if ( (int)(*length) <= 0 )
    {
        return NULL;
    }

    int i;

    for ( i = 0; i < (int)(*length) - 1 ; i++ )
    {
        memset(tmp, 0, sizeof(tmp));
        sprintf(tmp, "%d.", name[i]);
        strcat((char *)gOidBuf, (const char *)tmp);
    }
    sprintf(tmp, "%d", name[i]);
    strcat((char *)gOidBuf, (const char *)tmp);    

    return gOidBuf;
}

int SnmpMgrStrToOid(char *str, oid **name)
{
    long value1;
    char *endstr;   
    vector <int> v1;    // vector for storing oid name
    char *a_pos = str;


    for ( ;; )
    {
        value1 = strtoul(a_pos, &endstr, 10);
        if ( value1 == 0 )
            break;

        v1.push_back(value1);        
        a_pos = endstr + 1;
    }           

    int length = (int)v1.size();

    if ( length > 0 )
    {
        *name = (oid *)malloc(length  * sizeof(oid));
        for ( int i = 0; i < length; i++ )
        {
            (*name)[i] = v1[i];
        }
    }

    return length;
}


/*********************
 *
 *  System specific implementation functions
 *
 *********************/

/*
 * Define the callback function used in the areca_variables structure.
 * This is called whenever an incoming request refers to an object
 *  within this sub-tree.
 *
 * Four of the parameters are used to pass information in.
 * These are:
 *    vp      The entry from the 'areca_variables' array for the
 *             object being queried.
 *    name    The OID from the request.
 *    length  The length of this OID.
 *    exact   A flag to indicate whether this is an 'exact' request
 *             (GET/SET) or an 'inexact' one (GETNEXT/GETBULK).
 *
 * Four of the parameters are used to pass information back out.
 * These are:
 *    name     The OID being returned.
 *    length   The length of this OID.
 *    var_len  The length of the answer being returned.
 *    write_method   A pointer to the SET function for this object.
 *
 * Note that name & length serve a dual purpose in both roles.
 */

u_char         *
var_areca(struct variable *vp,
          oid * name,
          size_t * length,
          int exact, size_t * var_len, WriteMethod ** write_method)
{
    /*
     *  The result returned from this function needs to be a pointer to
     *    static data (so that it can be accessed from outside).
     *  Define suitable variables for any type of data we may return.
     */
    static char     string[4][256];    /* for ARECASTRING   */
    static oid      oid_ret[4][8]; /* for ARECAOBJECTID */
    static long     long_ret[4];   /* for everything else */       

    unsigned int    I;
    int             requestID = 1;  
    ARC_STATUS      stat = ARC_SUCCESS;
    char            SnmpMsgReturned[1024];
    int             SnmpMsgLen = 0;
    char            *result_str = NULL;
    int             result_len = 0;
    long            result = 0;
    oid             newname[MAX_OID_LEN];
    char            *szCommunity = gCommunity;
    int                 ret_oid[20];
    unsigned char       *var = NULL;
    unsigned char       tag = 0;    
    int         ret_var_len = 0;
    static char gOidBuf[64];


    *write_method = NULL;

    // Search for var bind name in the MIB           
    I = 0;

    // the location 8 of oid number is the subsystem OID,
    // and it is the controller number too, we just save it
    // for subsequent operations.
    unsigned int    CurrentCtrl = (unsigned int)name[8] - 1;


    // since we just have 1 copy of mib table, so we must replace 
    // the original subsystem oid with "1" to look it up in the mib table,
    // if finished we must restore the oid number to the original one.
    name[8] = 1;

    int cat = (int)name[9] - 1;    

    if ( (int)name[9] == 5000 )
    {
        return NULL;
    }

    if ( (int)name[9] == 6 && (int)name[13] == 63 )
    {
        oid startofeventoid[14] = {1,3,6,1,4,1,18928,1,1,6,1,1,2,0};
        memcpy((void *)name, (void *)startofeventoid, 14 * sizeof(oid));
    }


    size_t tmpint = (size_t)vp->namelen;
    SnmpUtilIdsToA(name, length, gOidBuf);

    stat = ar[CurrentCtrl].ArcSNMPWithOid( (int *)name, 
                                           (int *)length, 
                                           (unsigned char *)szCommunity, 
                                           ret_oid, 
                                           &tag, 
                                           &var, 
                                           (int *)&ret_var_len );
    if ( stat == ARC_SUCCESS )
    {
        SnmpUtilIdsToA(name, length, gOidBuf);

        // Save value in MIB
        if ( tag == ASN_INTEGER )
        {
            // restore the original subsystem oid            
            *var_len = ret_var_len;
            long_ret[CurrentCtrl] = *((long *)var);                   
            result = (long)(&long_ret[CurrentCtrl]);       
        }
        else if (tag == ASN_OCTET_STR)
        {
            result_str = (char *)var;
            result_len = ret_var_len;
            result_str[result_len] = 0;

            sprintf(string[CurrentCtrl],"%s", result_str);
            /*
             * Note that the assumption that the answer will be an
             *  integer does not hold true in this case, so the length
             *  of the answer needs to be set explicitly.           
             */
            *var_len = strlen(string[CurrentCtrl]);          

            result = (long)string[CurrentCtrl];
        }
        else
        {
            DebugPrint(log_fd, "Unsupported VarBind->value.asnType == %d\r\n", tag);
        }  
    }
    else
    {
        DebugPrint(log_fd, "Failed to ArcSnmp, state = 0x%X\r\n", stat);
    }

    Exit:   
    if (result == 0)
    {
        *length = 0;
    }

    // restore the original subsystem oid
    name[8] = CurrentCtrl + 1;

    return(u_char *)result;
}

void 
EventMonitor( unsigned int clientreg,
              void *clientarg)
{

    oid objid_enterprisetrap[] = { 1, 3, 6, 1, 4, 1, 18928, 1, 1};
    oid objid_rscreatelist[] = { 1, 3, 6, 1, 2, 1, 1, 1, 0};

    CArclib         *ctrl;
    ARC_STATUS      stat = ARC_SUCCESS;
    int             nCount = 0;             // total number of event objects we got	
    int             i;  
    int             nEventFlag = FLAG_NEW;  // the flag pass into ArcGetReqEventPage(...)
    int             nCurEvtLevel = EVT_SERIOUS;
    sSYSTEM_INFO    sysInfo;
    BYTE            ptr1[64];
    char            tmpStr[64];
    BOOL            gotEvent = FALSE;
    char            eventStrBuf[512];
    static  int CtrlIndex = 0;


    BYTE *eventWLevel[] ={
        (BYTE *)"URGENT",
        (BYTE *)"SERIOUS",
        (BYTE *)"WARNING",
        (BYTE *)"MESSAGE"
    };


    if (nTotalCtrl == 0)
        return;

    CtrlIndex = CtrlIndex % nTotalCtrl;

    ctrl = &ar[CtrlIndex];

    memset(ptr1, 0, sizeof(ptr1));
    memset(tmpStr, 0, sizeof(tmpStr));
    memset(eventStrBuf, 0, sizeof(eventStrBuf));
    // event object defined in event.h
    pEVENT_DATA pEventObject = (pEVENT_DATA)new sEVENT_DATA;    

    memset(pEventObject, 0, sizeof(sEVENT_DATA));

    stat = ctrl->ArcGetSysInfo(&sysInfo);
    if (stat != ARC_SUCCESS)
    {
        goto quit;
    }

    /* 
        We should call ArcSetTime() to correct the raid-subsystem time
    */
    /* 
        Call ArcGetReqEventPage() to get the required event page from raid subsystem.
        There are 4 event pages reside in the raid subsystem and each contains 16 event objects.			
    */
    stat = ctrl->ArcGetReqEventPage(nEventFlag, &nCount);
    if (stat == ARC_SUCCESS)
    {
        if (nCount != 0)
        {
            for ( i = nCount - 1; i >= 0; i--)
            {
                /*
                    Call ArcGetEventObject function to extract the individual event object from event page
                */
                if (ctrl->ArcGetEventObject( i, pEventObject ))
                {
                    /* 
                        Parse the event object we got
                    */
                    switch ((int)pEventObject->evtCategory)
                    {
                    /* 
                        The member "evtStr" of event object structure contains the string to indicate
                        which raidset owns this event object
                    */
                    case EVENT_RAIDSET:     // raidset event	
                        nCurEvtLevel = raidEventLevel[pEventObject->evtType];
                        strcpy( (char *)ptr1, htmRaidEvent[pEventObject->evtType]); 
                        memcpy(tmpStr, pEventObject->evtStr, 16);
                        sprintf((char *)eventStrBuf, "Controller#%d %s %s",CtrlIndex+1, tmpStr, ptr1);
                        gotEvent = TRUE;
                        break;                          
                        /* 
                            The member "evtStr" of event object structure contains the string to indicate
                            which volumeset owns this event object
                        */
                    case EVENT_VOLUMESET: // volumeset event
                        nCurEvtLevel = volEventLevel[pEventObject->evtType];
                        strcpy( (char *)ptr1, htmVolEvent[pEventObject->evtType]);  
                        memcpy(tmpStr, pEventObject->evtStr, 16);
                        sprintf((char *)eventStrBuf, "Controller#%d %s %s",CtrlIndex+1, tmpStr, ptr1);
                        gotEvent = TRUE;
                        break;

                    case EVENT_DEVICE:  // device event
                        nCurEvtLevel = devEventLevel[pEventObject->evtType];
                        sprintf(tmpStr, "IDE Channel #%2d", pEventObject->evtChannel + 1);
                        sprintf((char *)eventStrBuf, "Controller#%d %s %s",CtrlIndex+1, tmpStr, htmDevEvent[pEventObject->evtType] );
                        gotEvent = TRUE;
                        break;

                    case EVENT_HOST:    // host event
                        /* this is Host SCSI events */
                        nCurEvtLevel = hostEventLevel[pEventObject->evtType];
                        if ( sysInfo.gsiScsiHostChannels)
                        {
                            sprintf(tmpStr, "%s %s\n",
                                    eventCat[pEventObject->evtCategory], 
                                    htmScsiHostEvent[pEventObject->evtType]);
                        }
                        else
                        {
                            sprintf(tmpStr, "%s %s\n",
                                    eventCat[pEventObject->evtCategory], 
                                    htmIdeHostEvent[pEventObject->evtType]);
                        }
                        sprintf((char *)eventStrBuf, "Controller#%d %s",CtrlIndex+1, tmpStr);
                        gotEvent = TRUE;
                        break;

                    case EVENT_HW_MONITOR:  // hardware monitor event
                        nCurEvtLevel = hwEventLevel[pEventObject->evtType]; 
                        memcpy(tmpStr, eventCat[pEventObject->evtCategory], 16);
                        sprintf((char *)eventStrBuf, "Controller#%d %s %s",CtrlIndex+1, tmpStr, htmHwMonEvent[pEventObject->evtType] );
                        gotEvent = TRUE;
                        break;

                    case EVENT_NEW_83782D:
                        nCurEvtLevel = newEventLevel[pEventObject->evtType]; 
                        memcpy(tmpStr, pEventObject->evtStr, 16);
                        sprintf((char *)eventStrBuf, "Controller#%d %s %s",CtrlIndex+1, tmpStr, htmNewEventStr[pEventObject->evtType] );
                        gotEvent = TRUE;
                        break;
                    case EVENT_NO_EVENT:
//							printf("		No Event\n");	
                    default:
                        break;  



                    }// end switch
                }// end if

                if (gotEvent == TRUE)
                {
                    if (gEvt_level != EVT_DISABLE && nCurEvtLevel <= gEvt_level)
                    {
                        netsnmp_variable_list * vars = NULL;

                        int specificTrap = (int)pEventObject->evtCategory * 128 + (int)pEventObject->evtType;

                        snmp_varlist_add_variable(&vars, objid_rscreatelist, OID_LENGTH(objid_rscreatelist),
                                                  ASN_OCTET_STR, (u_char *)eventStrBuf, strlen(eventStrBuf));

                        send_enterprise_trap_vars(SNMP_TRAP_ENTERPRISESPECIFIC, specificTrap, objid_enterprisetrap,
                                                  OID_LENGTH(objid_enterprisetrap), vars);

                        snmp_free_varbind(vars);

                        DebugPrint(log_fd, "Ctrl[%d] issues the SNMP-Trap: %s\r\n", CtrlIndex, eventStrBuf);
                    }
                    // reset the goEvent flag
                    gotEvent = FALSE;
                }
            }// end for
        }//end if
    }// end if

    quit:

    // monitor next controller
    CtrlIndex++;

    if (pEventObject)
        delete pEventObject;
}

#ifdef __cplusplus
}
#endif




